/**
 * \file hamming.h
 *
 * \brief Mini librairy to manipulate binary Hamming codes parameters,
 *        for code based steganography.
 *
 * \author Morgan Barbier <morgan.barbier@ensicaen.fr>
 *
 * \version 1.0
 *
 * \date 2014-10-15
 *
 * \copyright This file is part of stegocurve.

 stegocurve is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 stegocurve is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with stegocurve. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __HAMMING_H__
#define __HAMMING_H__

#include "codes/code.h"
#include <math.h>

/**
 * \fn void init_Hamming_code(Code *C, const uint parameter);
 *
 * \brief initialize the code C with the parameters of a binary
 *        Hamming code of parameter 'parameter'. This initialization
 *        permits to use this code in the worst and average case.
 *
 * \param[out] C the code to initialize
 * \param[in] parameter of the binary Hamming code
 */
void init_Hamming_code(Code *C, const uint parameter);
/**
 * \fn void init_Hamming_codes(Code *C, const uint start, const uint nb);
 *
 * \brief initialize 'nb' codes in the array C with the parameters of binary
 *        Hamming codes of smallest parameter 'start'. This initialization
 *        permits to use these codes in the worst and average case.
 *
 * \param[out] C the code to initialize
 * \param[in] start the first parameter of a binary Hamming code
 * \param[in] nb the number of Hamming codes in the codes array C
 */
void init_Hamming_codes(Code *C, const uint start, const uint nb);

/**
 * \fn void Hamming_codes_WC(const char* filename, const uint start, const uint nb);
 *
 * \brief Create or erase the file filename, and print inside the worst cases
 *        of nb stegoschemes based on binary Hamming codes which has the 
 *        smallest parameter start.
 *
 * \param[in] filename the name of the output file
 * \param[in] start the smallest parameter of the binary Hamming codes
 * \param[in] nb the number of the binary Hamming codes treated
 */
void Hamming_codes_WC(const char* filename, const uint start, const uint nb);
/**
 * \fn void Hamming_codes_AC(const char* filename, const uint start, const uint nb);
 *
 * \brief Create or erase the file filename, and print inside the average cases
 *        of nb stegoschemes based on binary Hamming codes which has the 
 *        smallest parameter start.
 *
 * \param[in] filename the name of the output file
 * \param[in] start the smallest parameter of the binary Hamming codes
 * \param[in] nb the number of the binary Hamming codes treated
 */
void Hamming_codes_AC(const char* filename, const uint start, const uint nb);

#endif /* __HAMMING_H__ */
