/**
 * \file plotter.h
 *
 * \brief Header file to define general plotter. Here we define
 * a general structure for a plotter and some general functions
 * to manipulate this structure.
 *
 * \author Morgan Barbier <morgan.barbier@ensicaen.fr>
 *
 *
 * \version 1.1
 *
 * \date 2015-03-03
 *
 * \copyright This file is part of stegocurve.

 stegocurve is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 stegocurve is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with stegocurve. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __PLOTTER_H__
#define __PLOTTER_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tools/stegocurve_args.h"
#include "tools/definition.h"


/**
 * \struct PLOTTER plotter.h
 *
 * \brief Structure where is managed plotters.
 */
struct PLOTTER
{
  /** Pointer to pre procesing function [void () (struct PLOTTER *P)] */
  void (*pre_processing) (struct PLOTTER *P);
  /** Pointer to procesing function [void () (struct PLOTTER *P)] */
  void (*processing)     (struct PLOTTER *P);
  /** Pointer to post procesing function [void () (struct PLOTTER *P)] */
  void (*post_processing)(struct PLOTTER *P);
  /** File name of the intern file  [char*] */
  char* Plotter_filename;
  /** File pointer of the intern file [FILE*] */
  FILE* Plotter_file;
  /** File name of the output file  [char*] */
  char* output_filename;
  /** Structure which contains the association data files and titles [stegocurve_args*] */
  const stegocurve_args* data;
};
/**
 * \typedef Plotter
 *
 * equivalent to struct PLOTTER
 */
typedef struct PLOTTER Plotter;

/**
 * \fn void plotter_init(Plotter *P, void (*pre)(Plotter *P), void (*pro)(Plotter *P), void (*post)(Plotter *P), const char* Plotter_filename, const char* output_filename, const stegocurve_args *data);
 *
 * \brief General function for initialize any plotter.
 *
 * \param[in] P the plotter to initialize
 * \param[in] pre is the pointer to function to pre-processing function
 * \param[in] pro is the pointer to function to processing function
 * \param[in] post is the pointer to function to post-processing function
 * \param[in] Plotter_filename to initialize the field structure of the same name
 *            and Plotter_file.
 * \param[in] output_filename the file name of the result contained the
 *            result file
 * \param[in] data is the structure stegocurve_args, contained the association
 *            path and curve title
 */
void plotter_init(Plotter *P, void (*pre)(Plotter *P), void (*pro)(Plotter *P), void (*post)(Plotter *P), const char* Plotter_filename, const char* output_filename, const stegocurve_args *data);

/**
 * \fn void plotter_run(Plotter *P);
 *
 \brief General function to run the plotter P. That is run, in this order,
 *      pre-, processing and post.
 *
 * \param[in] P the plotter to run.
 */
void plotter_run(Plotter *P);

/**
 * \fn void plotter_free(Plotter *P);
 *
 * \brief Function to free the memory of the plotter gives in
 *        arguments.
 *
 * \param[in] P the plotter to desallocate.
 */
void plotter_free(Plotter *P);

#endif /* __PLOTTER_H__ */
