/**
 * \file stegocurve_args.h
 *
 * \brief Header file for create, free and manipulate the
 *        structure stegocurve_args. This structure is used
 *        to manage data files and their associated curve
 *        title.
 *
 * \author Morgan Barbier <morgan.barbier@ensicaen.fr>
 *
 * \version 1.1
 *
 * \date 2015-03-04
 *
 * \copyright This file is part of stegocurve.

 stegocurve is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 stegocurve is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with stegocurve. If not, see <http://www.gnu.org/licenses/>.

 */
#ifndef __STEGOCURVE_ARGS_H__
#define __STEGOCURVE_ARGS_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tools/definition.h"

/**
 * \struct STEGOCURVE_ARGS
 *
 * \brief Structure for manage data files and their associated
 *        title.
 */
struct STEGOCURVE_ARGS
{
  /** data is an array for associate the data files and cruve title [char**[2]] */
  char* (*data)[2];
  /** size size of the array data [uint] */
  uint size;
};
/** 
 * \typedef typedef struct STEGOCURVE_ARGS stegocurve_args;
 * \brief alias for the structure STEGOCURVE_ARGS
 */
typedef struct STEGOCURVE_ARGS stegocurve_args;


/**
 * \fn void stegocurve_args_init(stegocurve_args *A, const uint argc, const char **args);
 *
 * \brief Allocate and initialize the given stegocurve_args from the array args of
 *        size argc.
 *
 * \param[out] A the stegocurve_args to allocate and initialize
 * \param[in]  argc the total size of the array argv
 * \param[in]  args is array containing the path to data file and then
 *             the curve title
 */
void stegocurve_args_init(stegocurve_args *A, const uint argc, const char **args);
/**
 * \fn void stegocurve_args_init_size(stegocurve_args *A, const uint size);
 *
 * \brief Allocate the given stegocurve_args of size size
 *
 * \param[out] A the stegocurve_args to allocate
 * \param[in]  size the size of the array into the stegocurve_args
 */
void stegocurve_args_init_size(stegocurve_args *A, const uint size);
/**
 * \fn void stegocurve_args_free(stegocurve_args *A);
 *
 * \brief Desallocate the given stegocurve_args
 *
 * \param[in] A the stegocurve_args to desallocate
 */
void stegocurve_args_free(stegocurve_args *A);

/**
 * \fn uint stegocurve_args_get_size     (const stegocurve_args *A);
 *
 * \brief This function is a getter of the size of the given
 *        stegocurve_args
 *
 * \param[in] A the stegocurve_args to have the size
 * \return uint which is the size of array into A
 */
uint stegocurve_args_get_size     (const stegocurve_args *A);
/**
 * \fn char *stegocurve_args_get_datafile(const stegocurve_args *A, const uint i);
 *
 * \brief This function is a getter of the i-th path to the data file of the 
 *        stegocurve_args A
 *
 * \warning if i if out of bound, the NULL pointer will be returned.
 *
 * \param[in] A the stegocurve_args to have the piece of information
 * \param[in] i the number of needed data file.
 * \return Either the path to the i-th data file contained in A, either
 *         the NULL pointer if i is out of bound.
 */
char *stegocurve_args_get_datafile(const stegocurve_args *A, const uint i);
/**
 * \fn char *stegocurve_args_get_title   (const stegocurve_args *A, const uint i);
 *
 * \brief This function is a getter of the i-th path to the curve title of the 
 *        stegocurve_args A
 *
 * \warning if i if out of bound, the NULL pointer will be returned.
 *
 * \param[in] A the stegocurve_args to have the piece of information
 * \param[in] i the number of needed data file.
 * \return Either the i-th curve title contained in A, either the NULL 
 *         pointer if i is out of bound.
 */
char *stegocurve_args_get_title   (const stegocurve_args *A, const uint i);
/**
 * \fn void stegocurve_args_set_datafile (stegocurve_args *A, const uint i, const char *str);
 *
 * \brief This function is a setter of the i-th path to the data file of the 
 *        stegocurve_args A
 *
 * \param[in] A the stegocurve_args to set
 * \param[in] i the number of data file to set
 * \param[in] str the new path to data file to set
 */
void stegocurve_args_set_datafile (stegocurve_args *A, const uint i, const char *str);
/**
 * \fn void stegocurve_args_set_title    (stegocurve_args *A, const uint i, const char *str);
 *
 * \brief This function is a setter of the i-th curve title of the 
 *        stegocurve_args A
 *
 * \param[in] A the stegocurve_args to set
 * \param[in] i the number of data file to set
 * \param[in] str the new curve title to set
 */
void stegocurve_args_set_title    (stegocurve_args *A, const uint i, const char *str);

#endif /* __STEGOCURVE_ARGS_H__ */
